/*
    Copyright (C) 2005 Manuel Lausch

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Manuel Lausch
    Rudolfstr. 9
    75177 Pforzheim
    GERMANY
    software@manuellausch.de
*/

#include <iostream>
#include <fstream>
#include <queue>
#include <SDL.h>
#include <SDL_gfxPrimitives.h>
#include <SDL_thread.h>
#include <SDL_ttf.h>
#include "snake.h"

// using std::queue;

unsigned int snake::food = 0;

snake::snake(SDL_Surface *pD, playground *pP, SDL_mutex *pMut)
{
	pFont = TTF_OpenFont("./arial.ttf", 20);

	pDisplay = pD;
	pPlayground = pP;
	this->pMut = pMut;

	int tt = rand() % PLAYGROUND_WIDTH;

	// initial position of snake

	pPlayground->set_field(tt, 1, f_snake, 0x0000ffff);
	pPlayground->set_field(tt, 2, f_snake, 0x0000ffff);

	punkte = 0;

	x = tt;
	y = 2;
	default_direction = down;

	koordinate temp;
	temp.x = tt;
	temp.y = 1;
	snake_queue.push(temp);
	temp.y = 2;
	snake_queue.push(temp);

	speed = 50;
	color = 0x0000ffff;

	if(food == 0)
		generate_food();

	my_number = ++food;

}

snake::~snake()
{
//	SDL_mutexP(pMut);
//	SDL_mutexV(pMut);
}

int snake::step()
{
	SDL_mutexP(pMut);
	direction way = ways.get();

	if ((way == down && default_direction != up) || (way == right && default_direction != left) || (way == up && default_direction != down) || (way == left && default_direction != right))
		default_direction = way;

	switch (default_direction)
	{
		case up:
			if (0 == y) y = PLAYGROUND_HEIGHT;
			y--;
			break;

		case right:
			x++;
			if (x == PLAYGROUND_WIDTH) x = 0;
			break;

		case down:
			y++;
			if (y == PLAYGROUND_HEIGHT) y = 0;
			break;

		case left:
			if (0 == x) x = PLAYGROUND_WIDTH;
			x--;
			break;
	}


	if(pPlayground->get_field(x, y) == f_food)
	{
		generate_food();

		punkte++;
		{
			SDL_Rect drect;
			SDL_Surface *font_src;
			char titel[256];
			sprintf(titel,"Punkte: %d", punkte);
			SDL_Color color = {255,255,255};
			font_src = TTF_RenderText_Solid(pFont, titel, color);
			drect.x = (my_number - 1) * 120 + 20;
			drect.y = PLAYGROUND_HEIGHT*QUADSIZE + 10;
			drect.w = font_src->w;
			drect.h = font_src->h;
			boxColor(pDisplay, drect.x, drect.y, drect.x+drect.w, drect.y+drect.h, 0x000000ff);
			SDL_BlitSurface(font_src, NULL, pDisplay, &drect);
			SDL_Flip(pDisplay);
		}
	}
	else
	{
		pPlayground->del_field(snake_queue.front().x, snake_queue.front().y);
		snake_queue.pop();
	}

	koordinate temp;
	temp.x = x;
	temp.y = y;
	snake_queue.push(temp);


	if(pPlayground->get_field(x, y) == f_wall || pPlayground->get_field(x, y) == f_snake)
	{
		SDL_mutexV(pMut);
		return 1;
	}
	else
	{
		pPlayground->set_field(x, y, f_snake, get_color());
		SDL_mutexV(pMut);
		return 0;
	}
}

void snake::generate_food()
{
	srand(time(0));

	unsigned short int px, py;

	do
	{
		px = (unsigned short int) (rand() % PLAYGROUND_WIDTH);
		py = (unsigned short int) (rand() % PLAYGROUND_HEIGHT);
	} while (pPlayground->get_field(px, py) != f_empty);

	pPlayground->set_field(px, py, f_food, 0x00ff00ff);
}

int snake::start(void* arg)
{
	snake * player = (snake*) arg;
	while(player->step() == 0)
	{
		SDL_Delay(player->get_speed());
	}

	return 0;
}

unsigned short int snake::get_speed()
{
	return speed;
}

void snake::set_speed(unsigned short int arg)
{
	speed = arg;
}

Uint32 snake::get_color()
{
	return color;
}

void snake::set_color(Uint32 arg)
{
	color = arg;
}


queue_key::queue_key()
{
	begin = end = 0;
}

queue_key::~queue_key()
{
}

direction queue_key::get()
{
	if(begin == end)
		return old;

	direction way = ways[end];

	end++;
	if(end == 20) end = 0;

	return way;
}

void queue_key::push(direction way)
{
	unsigned short int temp;
	if(begin == 0)
		temp = 19;
	else
		temp = begin - 1;

	if(ways[temp] != way)
	{
		ways[begin] = way;

		begin++;
		if(begin == 20) begin = 0;
	}
}


playground::playground(SDL_Surface* pDisp)
{
	pDisplay = pDisp;
	for(int x = 0; x < PLAYGROUND_WIDTH; x++)
		for(int y = 0; y < PLAYGROUND_HEIGHT; y++)
			del_field(x, y);
}

playground::playground(SDL_Surface* pDisp, char* wall_name)
{
	pDisplay = pDisp;
	std::ifstream fin(wall_name);
	char ch;

	for(int y = 0; y < 50; y++)
		for(int x = 0; x < 50; x++)
		{
			fin.get(ch);

			if(ch == f_empty)
				del_field(x, y);
			if(ch == f_wall)
				set_field(x, y, f_wall, 0xffffffff);

		}

	fin.close();
}

playground::~playground()
{
}

void playground::set_field(short int x, short int y, field type, Uint32 color)
{
	SDL_Rect drect;

	grid[x][y] = type;
	if(type == f_food)
		filledCircleColor(pDisplay, x * QUADSIZE + QUADSIZE / 2 - 1, y * QUADSIZE + QUADSIZE / 2 - 1, (QUADSIZE - 1) / 2, color);
	else
		boxColor(pDisplay, x * QUADSIZE, y * QUADSIZE, x * QUADSIZE + QUADSIZE - 2, y * QUADSIZE + QUADSIZE - 2, color);
	drect.x = x * QUADSIZE;
	drect.y = y * QUADSIZE;
	drect.w = drect.h = QUADSIZE;
	SDL_UpdateRects(pDisplay, 1, &drect);
}

void playground::del_field(short int x, short int y)
{
	SDL_Rect drect;

	grid[x][y] = f_empty;

	boxColor(pDisplay, x * QUADSIZE, y * QUADSIZE, x * QUADSIZE + QUADSIZE - 1, y * QUADSIZE + QUADSIZE - 1, 0x000000ff);
	drect.x = x * QUADSIZE;
	drect.y = y * QUADSIZE;
	drect.w = drect.h = QUADSIZE;
	SDL_UpdateRects(pDisplay, 1, &drect);
}

field playground::get_field(short int x, short int y)
{
//	std::cout << grid[x][y] << "  " << x << "  " << y << std::endl;
	return grid[x][y];
}
